#!/bin/env dls-python

import read_config as cfg
import importlib
import socket
import time
import os, sys, argparse
import logging


class RunTests(object):
    def __init__(self, configpath, hostname, cmdport):
        self.file_logger = logging.getLogger('filelog')
        self.console_logger = logging.getLogger('consolelog')
        self.hostname = hostname
        self.config = self.get_config(configpath)
        self.test_modules = self.import_modules()
        try:
            self.cmdsock = self.setup_connection(hostname, cmdport)
        except socket.error as e:
            raise
            self.console_logger.error(e)

    def setup_connection(self, hostname, cmdport):
        try:
            cmdsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            cmdsock.connect((hostname, cmdport))
            cmdsock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
            cmdsock.settimeout(1)
        except socket.error:
            self.console_logger.error(
                "Problem with connection to {}".format(self.hostname))
            raise
        return cmdsock

    def get_config(self, path):
        cfgread = cfg.ConfigReader()
        return cfgread.parse_config_file(path)

    def import_modules(self):
        modules = {}
        for key, name in self.config.iteritems():
            try:
                m_name_full = 'tests.' + name[0][0]
                modules[name[0][0]] = importlib.import_module(m_name_full)
            except ImportError as e:
                self.console_logger.error(
                    "MODULE DOES NOT EXIST, SKIPPING:  {}".format(name))
                del self.config[name[0][0]]
        return modules

    def run_tests(self):
        cls_list = []
        for name, config_list in self.config.items():
            test_cls = getattr(self.test_modules[name], name)
            for config in config_list:
                #restore panda state to make sure we have a known start point
                os.system(
                    "./load-state {} base.state".format(self.hostname))
                self.file_logger.info('-- {:<7} {} {}'.format(
                                                            name.upper(),
                                                            config[1],
                                                            ''.join(['-']*32)))
                tst = test_cls(self.cmdsock, self.hostname, config[1])
                cls_list.append(tst)
                tst.run()
                tst.get_summary()
        self.print_summary(cls_list)

    def print_summary(self, cls_list):
        self.console_logger.info('-- SUMMARY {}'.format(''.join(['-'] * 34)))
        self.console_logger.info(
            '{:<12}{:<12}{:<11}{:<10}'.format('TEST', 'TYPE', 'BLOCK', 'ERRORS'))
        self.console_logger.info('{}'.format(''.join(['-'] * 45)))
        for cls in cls_list:
            summary = cls.get_summary()
            for item in summary:
                self.console_logger.info(item)

def setup_logging(logPath):
    file_format = "%(asctime)s [%(levelname)-5s] [%(name)-13s]  %(message)s"
    console_format = "%(asctime)s [%(levelname)-5s] %(message)s"
    logging.basicConfig(level=logging.DEBUG,
                        format=file_format,
                        filename=logPath)
    #set up console logger
    logFormatter = logging.Formatter(console_format)
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(logFormatter)
    logging.getLogger('consolelog').addHandler(console_handler)


def view_log(log_file_path):
    inputy = '.'
    while inputy.strip('\n') not in ['y', 'n']:
        print "\nView logfile: {} [y/n]".format(log_file_path)
        inputy = raw_input()
    if inputy.strip('\n') in ['y']:
        f = open(log_file_path, 'r')
        file_contents = f.read()
        print (file_contents)
        f.close()


def main(argv):
    timestr = time.strftime("%Y%m%d-%H%M%S")
    log_dir = 'log'
    log_path = os.path.join('log', "{}.log".format(timestr))
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)
    setup_logging(log_path)

    # Argument processing
    parser = argparse.ArgumentParser(
        description = 'Run PandA hardware I/O tests')
    parser.add_argument(
        '--port', '-p', default=8888, help='Configuration port number')
    parser.add_argument(
        'hostname', help = 'Name or IP address of Panda device')
    args = parser.parse_args()

    # try:
    rt = RunTests('config', args.hostname, int(args.port))
    try:
        rt.run_tests()
    except Exception as e:
        print "EXCEPTION", e
        pass
    view_log(log_path)
    # except:
    #     print "CONNECTION ERROR"

if __name__ == '__main__':
    main(sys.argv[1:])
